package starterCode;


import java.io.BufferedReader;
import java.io.FileReader;

import tests.NetChecker2;

import junit.framework.TestCase;
import junit.textui.TestRunner;


/***************************************************
 Assignment 3   - Spring 2008
 Original Code Copied from: Mikhail Afanasyev
 Course: CS 182
 
 Assignment 3: Backpropagation
 
 Modified by Leon Barrett
 **************************************************/

/**
 * Class for testing backpropagation networks.
 */
public class TesterPart2 extends TestCase {
	Net n;
	NetChecker2 checker;
	NetParams p;
	
	void runNetParams() {
		checker = new NetChecker2(p);
		n = new Net(p,checker);
		n.train();
	}
	
	/**
	 * Test a network learning the AND function.
	 */
	public void testAnd() {
		int[][][] data = loadTrainingDataFromFile("and.data");
		p = new NetParams();
		p.trainingData = data[0];
		p.trainingTargets = data[1];
		p.numInputs = data[0][0].length;
		p.numOutputs = data[1][0].length;
		p.numHidden = 1;
		p.numHidLayers = 1;
		runNetParams();
	}

	/** Test a network learnign the OR function. */
	public void testOr() {
		int[][][] data = loadTrainingDataFromFile("or.data");
		p = new NetParams();
		p.trainingData = data[0];
		p.trainingTargets = data[1];
		p.numInputs = data[0][0].length;
		p.numOutputs = data[1][0].length;
		p.numHidden = 1;
		p.numHidLayers = 1;
		runNetParams();
	}

	/**
	 * Create a network for learning the SAME function.
	 */
	public void testSame() {
		int[][][] data = loadTrainingDataFromFile("same.data");
		p = new NetParams();
		p.trainingData = data[0];
		p.trainingTargets = data[1];
		p.numInputs = data[0][0].length;
		p.numOutputs = data[1][0].length;
		p.numHidden = 2;
		p.numHidLayers = 1;
		runNetParams();
	}

	/**
	 * Create a network for leaning N-input, M-hidden node auto encoder.
	 */
	static public NetParams createAutoCoder(int inputs, int hidden) {
		int patterns[][] = new int[inputs][inputs];
		for (int i = 0; i < inputs; i++) {
			patterns[i] = new int[inputs];
			for (int j = 0; j < inputs; j++)
				patterns[i][j] = (i == j) ? 1 : 0;
		}
		NetParams p = new NetParams();
		p.trainingData = patterns;
		p.trainingTargets = patterns;
		p.numInputs = inputs;
		p.numOutputs = inputs;
		p.numHidden = hidden;
		p.numHidLayers = 1;
		return p;
	};
	
	/** Test a simple auto-encoder. */
	public void testAutoCoder4() {
		p = createAutoCoder(4, 2);
		runNetParams();
	}
	
	/** Test a whole bunch of networks! */
	public void testOtherNets() {
			String[] learnList = { "and.data", "or.data", "xor.data", "same.data",
				"auto8.data", "multi.data" };
		double[] lrList = { 0.02, 0.1 };
		double[] momList = { 0.0, 0.7 };
		int hidlayers = 1;
		int[] nhidList = { 2, 3 };
		for (String learn : learnList) {
			for (double mom : momList) {
				for (double lr : lrList) {
					for (int nhid : nhidList) {
						if (nhid == 2 && learn.equals("auto8.data"))
							continue;
						for (NetParams.TrainType trainType : NetParams.TrainType
								.values()) {
							int[][][] data = loadTrainingDataFromFile(learn);
							NetParams params = new NetParams();
							params.trainingData = data[0];
							params.trainingTargets = data[1];
							params.numHidLayers = hidlayers;
							params.numHidden = nhid;
							params.trainType = trainType;
							params.numInputs = data[0][0].length;
							params.numOutputs = data[1][0].length;
							params.numEpochs = 200;
							params.learningRate = lr;
							params.momentum = mom;
							params.errorCriterion = 0.01;

							NetChecker2 checker = new NetChecker2(params);
							Net n = new Net(params, checker);

							n.train();
						}
					}
				}
			}
		}
	}

	/**
	 * The function should be called as: 
	 * % java TesterPart2 datafile ne lr mom ec hidLayers nhid
	 * where:
	 * 		datafile contains the data to be learned 
	 * 		ne is the number of epochs
	 * 		lr is the learning rate
	 * 		mom is the momentum 
	 * 		ec is the error criterion 
	 * 		hidLayers is the number of hidden layers
	 * 		nhid is the number of neurons per hidden layer
	 * @throws ClassNotFoundException 
	 */
	public static void main(String[] argv) throws ClassNotFoundException {
		if (argv.length != 8 && argv.length != 0) {
			System.err.println("Invalid argument count");
			System.err
					.println("In order to run automated tests, the function should be called as:");
			System.err.println("    % java TesterPart2");
			System.err.println(" or to run on a data set as");
			System.err
					.println("    % java TesterPart2 datafile ne lr mom ec hidLayers nhid trainType");
			System.err.println(" where:");
			System.err.println("    datafile contains the data to be learned");
			System.err.println("    ne is the number of epochs");
			System.err.println("    lr is the learning rate");
			System.err.println("    mom is the momentum");
			System.err.println("    ec is the error criterion");
			System.err.println("    hidLayers is the number of hidden layers");
			System.err
					.println("    nhid is the number of neurons per hidden layer");
			System.err
					.println("    trainType is the training type, either Batch or Stochastic");
			return;
		}
		if (argv.length == 0) {
			TestRunner.run(Class.forName("starterCode.TesterPart2"));
		} else {
			// parse command-line
			int[][][] data = loadTrainingDataFromFile(argv[0]);
			NetParams params = new NetParams();
			params.trainingData = data[0];
			params.trainingTargets = data[1];
			params.numHidLayers = Integer.parseInt(argv[5]);
			params.numHidden = Integer.parseInt(argv[6]);
			params.trainType = NetParams.TrainType.Stochastic;
			if (argv[7].equals("Batch"))
				params.trainType = NetParams.TrainType.Batch;
			params.numInputs = data[0][0].length;
			params.numOutputs = data[1][0].length;
			params.numEpochs = Integer.parseInt(argv[1]);
			params.learningRate = Double.parseDouble(argv[2]);
			params.momentum = Double.parseDouble(argv[3]);
			params.errorCriterion = Double.parseDouble(argv[4]);

			NetChecker2 checker = new NetChecker2(params);
			Net n = new Net(params, checker);

			n.train();
		}
	}

	/**
	 * Load training data out of a file.
	 * 
	 * @param filename
	 */
	public static int[][][] loadTrainingDataFromFile(String filename) {
		int[][] trainingData = null;
		int[][] trainingTargets = null;
		try {
			BufferedReader in = new BufferedReader(new FileReader(filename));
			String s;
			int state = 0, count = 0;
			int nData = 0, nIn = 0, nOut = 0;
			while ((s = in.readLine()) != null) {
				String[] parts = s.split(" ");
				switch (state) {
				case 0:
					if (parts[0].equals("DATA_DESCRIPTION"))
						state = 1;
					break;
				case 1:
					nData = Integer.parseInt(parts[0]);
					nIn = Integer.parseInt(parts[1]);
					nOut = Integer.parseInt(parts[2]);
					trainingData = new int[nData][nIn];
					trainingTargets = new int[nData][nOut];
					state = 2;
					break;
				case 2:
					if (parts[0].equals("DATA"))
						state = 3;
					break;
				case 3:
					if (parts.length != nIn + nOut + 1)
						throw new Exception("failed reading file");
					trainingData[count] = parseNInts(parts, 0, nIn);
					trainingTargets[count] = parseNInts(parts, nIn + 1, nOut);
					count++;
					break;
				}
			}
		} catch (Exception e) {
			throw new Error("failed reading file " + filename);
		}
		int[][][] result = { trainingData, trainingTargets };
		return result;
	}

	/**
	 * Parse part of an array of integers.
	 * 
	 * @param arr
	 *            The array to parse
	 * @param start
	 *            The element in the array to start with
	 * @param count
	 *            The number of elements to parse
	 * @return the parsed integers
	 */
	public static int[] parseNInts(String[] arr, int start, int count) {
		int[] d = new int[count];
		for (int i = 0; i < count; i++) {
			d[i] = Integer.parseInt(arr[i + start]);
		}
		return d;
	}
}